from prowler.providers.azure.lib.audit_info.audit_info import azure_audit_info
from prowler.providers.azure.services.cosmosdb.cosmosdb_service import CosmosDB

cosmosdb_client = CosmosDB(azure_audit_info)



from prowler.providers.azure.lib.audit_info.audit_info import azure_audit_info
from prowler.providers.azure.services.keyvault.keyvault_service import KeyVault

keyvault_client = KeyVault(azure_audit_info)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.lib.scan_filters.scan_filters import is_resource_filtered
from prowler.providers.aws.lib.service.service import AWSService


################## CognitoIDP
class CognitoIDP(AWSService):
    def __init__(self, audit_info):
        super().__init__("cognito-idp", audit_info)
        self.user_pools = {}
        self.__threading_call__(self.__list_user_pools__)
        self.__describe_user_pools__()
        self.__get_user_pool_mfa_config__()

    def __list_user_pools__(self, regional_client):
        logger.info("Cognito - Listing User Pools...")
        try:
            user_pools_paginator = regional_client.get_paginator("list_user_pools")
            for page in user_pools_paginator.paginate(MaxResults=60):
                for user_pool in page["UserPools"]:
                    arn = f"arn:{self.audited_partition}:cognito-idp:{regional_client.region}:{self.audited_account}:userpool/{user_pool['Id']}"
                    if not self.audit_resources or (
                        is_resource_filtered(arn, self.audit_resources)
                    ):
                        try:
                            self.user_pools[arn] = UserPool(
                                id=user_pool["Id"],
                                arn=arn,
                                name=user_pool["Name"],
                                region=regional_client.region,
                                last_modified=user_pool["LastModifiedDate"],
                                creation_date=user_pool["CreationDate"],
                                status=user_pool.get("Status", "Disabled"),
                            )
                        except Exception as error:
                            logger.error(
                                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                            )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __describe_user_pools__(self):
        logger.info("Cognito - Describing User Pools...")
        try:
            for user_pool in self.user_pools.values():
                try:
                    user_pool_details = self.regional_clients[
                        user_pool.region
                    ].describe_user_pool(UserPoolId=user_pool.id)["UserPool"]
                    user_pool.password_policy = user_pool_details.get(
                        "Policies", {}
                    ).get("PasswordPolicy", {})
                    user_pool.deletion_protection = user_pool_details.get(
                        "DeletionProtection", "INACTIVE"
                    )
                    user_pool.advanced_security_mode = user_pool_details.get(
                        "UserPoolAddOns", {}
                    ).get("AdvancedSecurityMode", "OFF")
                    user_pool.tags = [user_pool_details.get("UserPoolTags", "")]
                except Exception as error:
                    logger.error(
                        f"{user_pool.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
        except Exception as error:
            logger.error(
                f"{user_pool.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __get_user_pool_mfa_config__(self):
        logger.info("Cognito - Getting User Pool MFA Configuration...")
        try:
            for user_pool in self.user_pools.values():
                try:
                    mfa_config = self.regional_clients[
                        user_pool.region
                    ].get_user_pool_mfa_config(UserPoolId=user_pool.id)
                    if mfa_config["MfaConfiguration"] != "OFF":
                        user_pool.mfa_config = MFAConfig(
                            sms_authentication=mfa_config.get(
                                "SmsMfaConfiguration", {}
                            ),
                            software_token_mfa_authentication=mfa_config.get(
                                "SoftwareTokenMfaConfiguration", {}
                            ),
                            status=mfa_config["MfaConfiguration"],
                        )
                except Exception as error:
                    logger.error(
                        f"{user_pool.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
        except Exception as error:
            logger.error(
                f"{user_pool.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class MFAConfig(BaseModel):
    sms_authentication: Optional[dict]
    software_token_mfa_authentication: Optional[dict]
    status: str


class UserPool(BaseModel):
    id: str
    arn: str
    name: str
    region: str
    advanced_security_mode: str = "OFF"
    deletion_protection: str = "INACTIVE"
    last_modified: datetime
    creation_date: datetime
    status: str
    password_policy: Optional[dict]
    mfa_config: Optional[MFAConfig]
    tags: Optional[list] = []



from prowler.providers.aws.lib.audit_info.audit_info import current_audit_info
from prowler.providers.aws.services.cognito.cognito_service import CognitoIDP

cognito_idp_client = CognitoIDP(current_audit_info)



import sys

import yaml
from jsonschema import validate

from prowler.config.config import valid_severities
from prowler.lib.logger import logger

custom_checks_metadata_schema = {
    "type": "object",
    "properties": {
        "Checks": {
            "type": "object",
            "patternProperties": {
                ".*": {
                    "type": "object",
                    "properties": {
                        "Severity": {
                            "type": "string",
                            "enum": valid_severities,
                        }
                    },
                    "required": ["Severity"],
                    "additionalProperties": False,
                }
            },
            "additionalProperties": False,
        }
    },
    "required": ["Checks"],
    "additionalProperties": False,
}


def parse_custom_checks_metadata_file(provider: str, parse_custom_checks_metadata_file):
    """parse_custom_checks_metadata_file returns the custom_checks_metadata object if it is valid, otherwise aborts the execution returning the ValidationError."""
    try:
        with open(parse_custom_checks_metadata_file) as f:
            custom_checks_metadata = yaml.safe_load(f)["CustomChecksMetadata"][provider]
            validate(custom_checks_metadata, schema=custom_checks_metadata_schema)
        return custom_checks_metadata
    except Exception as error:
        logger.critical(
            f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
        )
        sys.exit(1)


def update_checks_metadata(bulk_checks_metadata, custom_checks_metadata):
    """update_checks_metadata returns the bulk_checks_metadata with the check's metadata updated based on the custom_checks_metadata provided."""
    try:
        # Update checks metadata from CustomChecksMetadata file
        for check, custom_metadata in custom_checks_metadata["Checks"].items():
            check_metadata = bulk_checks_metadata.get(check)
            if check_metadata:
                bulk_checks_metadata[check] = update_check_metadata(
                    check_metadata, custom_metadata
                )
        return bulk_checks_metadata
    except Exception as error:
        logger.critical(
            f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
        )
        sys.exit(1)


def update_check_metadata(check_metadata, custom_metadata):
    """update_check_metadata updates the check_metadata fields present in the custom_metadata and returns the updated version of the check_metadata. If some field is not present or valid the check_metadata is returned with the original fields."""
    try:
        if custom_metadata:
            for attribute in custom_metadata:
                try:
                    setattr(check_metadata, attribute, custom_metadata[attribute])
                except ValueError:
                    pass
    finally:
        return check_metadata



import re

from prowler.lib.check.models import Check_Report_AWS


def check_cloudwatch_log_metric_filter(
    metric_filter_pattern: str,
    trails: list,
    metric_filters: list,
    metric_alarms: list,
    report: Check_Report_AWS,
):
    # 1. Iterate for CloudWatch Log Group in CloudTrail trails
    log_groups = []
    for trail in trails.values():
        if trail.log_group_arn:
            log_groups.append(trail.log_group_arn.split(":")[6])
    # 2. Describe metric filters for previous log groups
    for metric_filter in metric_filters:
        if metric_filter.log_group in log_groups:
            if re.search(metric_filter_pattern, metric_filter.pattern, flags=re.DOTALL):
                report.resource_id = metric_filter.log_group
                report.resource_arn = metric_filter.arn
                report.region = metric_filter.region
                report.status = "FAIL"
                report.status_extended = f"CloudWatch log group {metric_filter.log_group} found with metric filter {metric_filter.name} but no alarms associated."
                # 3. Check if there is an alarm for the metric
                for alarm in metric_alarms:
                    if alarm.metric == metric_filter.metric:
                        report.status = "PASS"
                        report.status_extended = f"CloudWatch log group {metric_filter.log_group} found with metric filter {metric_filter.name} and alarms set."
                        break

    return report



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.network.network_client import network_client


class network_watcher_enabled(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []
        nw_locations = []
        for subscription, network_watchers in network_client.network_watchers.items():
            for network_watcher in network_watchers:
                nw_locations.append(network_watcher.location)
        for subscription, locations in network_client.locations.items():
            for location in locations:
                report = Check_Report_Azure(self.metadata())
                report.subscription = subscription
                report.resource_name = "Network Watcher"
                report.resource_id = f"/subscriptions/{subscription}/providers/Microsoft.Network/networkWatchers/{location}"
                if location not in nw_locations:
                    report.status = "FAIL"
                    report.status_extended = f"Network Watcher is not enabled for the location {location} in subscription {subscription}."
                    findings.append(report)
                else:
                    report.status = "PASS"
                    report.status_extended = f"Network Watcher is enabled for the location {location} in subscription {subscription}."
                    findings.append(report)

        return findings



from azure.identity import AzureAuthorityHosts
from msrestazure.azure_cloud import (
    AZURE_CHINA_CLOUD,
    AZURE_GERMAN_CLOUD,
    AZURE_US_GOV_CLOUD,
)


def get_regions_config(region):
    allowed_regions = {
        "AzureCloud": {
            "authority": None,
            "base_url": "https://management.azure.com",
            "credential_scopes": ["https://management.azure.com/.default"],
        },
        "AzureChinaCloud": {
            "authority": AzureAuthorityHosts.AZURE_CHINA,
            "base_url": AZURE_CHINA_CLOUD.endpoints.resource_manager,
            "credential_scopes": [
                AZURE_CHINA_CLOUD.endpoints.resource_manager + "/.default"
            ],
        },
        "AzureUSGovernment": {
            "authority": AzureAuthorityHosts.AZURE_GOVERNMENT,
            "base_url": AZURE_US_GOV_CLOUD.endpoints.resource_manager,
            "credential_scopes": [
                AZURE_US_GOV_CLOUD.endpoints.resource_manager + "/.default"
            ],
        },
        "AzureGermanCloud": {
            "authority": AzureAuthorityHosts.AZURE_GERMANY,
            "base_url": AZURE_GERMAN_CLOUD.endpoints.resource_manager,
            "credential_scopes": [
                AZURE_GERMAN_CLOUD.endpoints.resource_manager + "/.default"
            ],
        },
    }
    return allowed_regions[region]
